"""\
Simple contact form extension for a Flows app.
"""

from cgi import escape
import logging

from bn import AttributeDict, HTMLFragment, str_dict
from configconvert import handle_option_error, handle_section_error,\
   unicodeToObject
from flows.provider import start_flow, ServiceProvider
from conversionkit import Conversion, toDictionary, set_error, \
   chainConverters, chainPostConverters, toListOf
from formbuild import Form
from formconvert import multiDictToDict
from mail.service import MailService
from nestedrecord import decodeNestedRecord, encode_error
import recaptcha.client.captcha
from recordconvert import toRecord, toListOfRecords
from stringconvert import unicodeToUnicode, unicodeToInteger
from stringconvert.email import listOfEmails, unicodeToEmail
from httpkit.service.input import HTTPInputService
from dreamweavertemplate.service import DreamweaverTemplateService

log = logging.getLogger(__name__)

#
# FormBuild forms
#

class CaptchaForm(Form):
    """\
    A custom FormBuild form with support for ``captcha`` fields as long as
    they are given the name ``'captcha'``.
    """

    def captcha(self, name, public_key, attributes=None):
        """Name gets ignored"""
        if name != 'captcha':
            raise Exception("The name field must be 'captcha', not %r"%name)
        if attributes is not None:
            raise Exception(attributes)
        log.info('Using public key %s', public_key)
        return """
<div id="recaptcha_widget" style="display:none">
    <span class="recaptcha_only_if_image">
        Type the two words in the picture below:
    </span>
    <span class="recaptcha_only_if_audio">
        Type the numbers you hear:
    </span>
    <div id="recaptcha_image"></div>
    <div class="recaptcha_only_if_incorrect_sol" style="color:red">
        Incorrect please try again
    </div>
    <input type="text" id="recaptcha_response_field" 
       name="recaptcha_response_field" size="38" />
    <p style="margin-top: 3px;">
        <small>
            Typing these words helps digitise books as well as preventing 
            spam
            <br />
            <a href="http://recaptcha.net/">reCAPTCHA</a> options:
            <a href="javascript:Recaptcha.reload()">Reload</a>
            <span class="recaptcha_only_if_image"> | 
                <a href="javascript:Recaptcha.switch_type('audio')">Audio</a>
            </span>
            <span class="recaptcha_only_if_audio"> | 
                <a href="javascript:Recaptcha.switch_type('image')">Image</a>
            </span>
            | <a href="javascript:Recaptcha.showhelp()">Help</a>
        </small>
    </p>
</div>
""" + recaptcha.client.captcha.displayhtml(public_key)

#
# Converters
#

def reCaptcha():
    def reCaptcha_post_converter(conversion, flow):
        if conversion.successful:
            challenge = conversion.result['recaptcha_challenge_field']
            response_ = conversion.result['recaptcha_response_field']
            private_key = flow.config.ext.private_key
            log.info('Using private key %r', private_key)
            rc = recaptcha.client.captcha.submit(
                challenge,
                response_,
                private_key,
                flow.http.environ.get('REMOTE_ADDR', ''),
            )
            if not rc.is_valid:
                set_error(
                    conversion,
                    "The words you entered were incorrect, please try again"
                )
    return reCaptcha_post_converter

def contactResults():
    def contactResults_post_converter(conversion, flow):
        if conversion.successful:
            to = flow.config.ext.to[conversion.result['to']]
            conversion.result['to_email'] = to['email']
            conversion.result['to_name'] = to['name']
    return contactResults_post_converter

def validToPosition():
    def validToPosition_converter(conversion, flow):
        converter = unicodeToInteger(min=0, max=len(flow.config.ext.to)-1)
        converter(conversion, flow)
    return validToPosition_converter

contact_form_converter = chainPostConverters(
    toRecord(
        filter_extra_fields = False,
        missing_or_empty_errors = {
            'from_name': 'Please enter your name',
            'from_email': 'Please enter your email address',
            'message': 'Please enter the message you want to send',
            'recaptcha_response_field': \
               'You haven\'t entered the two words below',
        },
        converters = dict(
            to = validToPosition(),
            from_name = unicodeToUnicode(),
            from_email = unicodeToEmail(),
            message = unicodeToUnicode(),
            recaptcha_response_field = unicodeToUnicode(),
        )
    ),
    reCaptcha(),
    contactResults(),
)

#
# Event Handlers
#

def on_generate_head_html(flow):
    return """\
<style type="text/css">
table.formbuild span.error-message, table.formbuild div.error, table.formbuild span.required {
    font-weight: bold;
    color: #f00;
}
table.formbuild span.small {
    font-size: 85%%;
}
table.formbuild form {
    margin-top: 20px;
}
table.formbuild form table td {
    padding-bottom: 3px;
}
table.formbuild label {
    font-weight: bold;
}
table.formbuild td.label {
    text-align: right;
}
#recaptcha_image {
    padding: 10px;
}
</style>
<script>
var RecaptchaOptions = {
   theme: 'custom',
   lang: 'en',
   custom_theme_widget: 'recaptcha_widget'
};
</script>"""

def on_generate_form_html(flow):
    form = flow.ext.form
    to = [(i, to['name']) for i, to in enumerate(flow.config.ext.to)]
    fragment = HTMLFragment()
    fragment.safe(form.start_with_layout(''))
    if len(to)>1:
        fragment.safe(
            form.field(
                label='To',
                type='dropdown', 
                name='to', 
                required=True,
            )
        )
    fragment.safe(
        form.field(
            label='Your Name',
            type='text', 
            name='from_name', 
            attributes=dict(size=38),
            required=True,
        )
    )
    fragment.safe(
        form.field(
            label='Your Email',
            type='text', 
            name='from_email',
            attributes=dict(size=38),
            required=True,
        )
    )
    fragment.safe(     
        form.field(    
            label='Message',
            type='textarea', 
            name='message', 
            attributes=dict(rows=7, cols=40),
            required=True,
        )
    )
    fragment.safe(
        form.field(
            name='captcha',
            label='Word verification',
            type='captcha',
            args=dict(public_key=flow.config.ext.public_key),
            required=True,
        )
    )
    actions = form.submit(name='submit', value='Send')
    if not len(to)>1:
        actions += form.hidden(name="to")
    fragment.safe(form.action_bar(actions))
    fragment.safe(form.end_with_layout())
    return fragment.getvalue()
    
def on_valid_data(flow, result):   
    """\
    The default handler for a successfully completed form.
                     
    Your implementation is expected to handle the contact form in some way,
    presumably by sending an email and then returning an encoded string 
    representing the) HTML to return to the browser to display a on_valid_data 
    message. 

    The ``flow`` argument has the following objects which might be of 
    particular use:
    """
    flow.provider.start('mail')
    flow.mail.send(
        message = result.message,
        to = '%s <%s>'%(result.to_name, result.to_email),
        from_email = result.from_email,
        from_name = result.from_name,
        subject = flow.config.ext.subject,
    )
    return flow.template.render(
        flow.config.ext.template, 
        {
            'doctitle': '<title>Contact Form</title>',
            'heading': 'Contact Form',
            'content': (
                '<p>The message below has been successfully sent:</p>'
                '<pre>'+escape(result.message)+'</pre>'
            ),
            'head': on_generate_head_html(flow),
        } 
    )

def on_new_form(flow):
    """\
    The default handler for a on_new_form form. 

    Your implementation is expected to 
    return an encoded string representing the HTML to return to the browser
    to display a on_new_form contact form. The ``flow`` argument has the following
    objects which might be of particular use:
    """
    return flow.template.render(
        flow.config.ext.template, 
        {
            'doctitle': '<title>Contact Form</title>',
            'heading': 'Contact Form',
            'content': (
                '<p>Please complete the following form to '
                'send a message.</p>'+\
                flow.ext.on_generate_form_html(flow)
            ),
            'head': flow.ext.on_generate_head_html(flow),
        }
    )

def on_validation_error(flow, form):
    """\
    The default handler for repopulating form containing errors.

    Your implementation is expected to
    return an encoded string representing the HTML to return to the browser
    to display a on_new_form contact form. The ``flow`` argument has the following
    objects which might be of particular use:
    """
    return flow.template.render(
        flow.config.ext.template,
        {
            'doctitle': '<title>Contact Form</title>',
            'heading': 'Contact Form',
            'content': (
                '<p class="error-message">Please correct the errors.</p>'+\
                on_generate_form_html(flow)
            ),
            'head': on_generate_head_html(flow),
        }
    )

#
# Extension
#

def config(flow, name):
    if not flow.config.option_group.has_key(name):
        raise handle_section_on_validation_error(
            flow, 
            name, 
            "'%s.to', '%s.private_key' and '%s.public_key'"%(
                name, 
                name, 
                name
            )
        )
    to_unicode = unicodeToUnicode()
    to_object = unicodeToObject()
    contact = toRecord(
        empty_errors = {
            'email': 'Please enter a value',
        },
        converters = dict(
            email = unicodeToEmail(),
            name = to_unicode,
        )
    )
    converter = chainConverters(
        decodeNestedRecord(),
        toRecord(
            missing_errors = {
                'private_key': 'No private key was found',
                'public_key': 'No public key was found',
                'to': 'No contact emails were specified', 
            },
            missing_defaults = {
                'template': 'main.dwt',
                'subject': 'Contact form message',
            },
            empty_errors = {
                'private_key': 'No private key was found',
                'public_key': 'No public key was found',
            },
            converters = dict(
                to = toListOfRecords(contact),
                template = to_unicode,
                private_key = to_unicode,
                public_key = to_unicode,
                subject = to_unicode,
            ),
        ),
    )
    conversion = Conversion(flow.config.option_group[name]).perform(
        converter,
        flow,
    )
    if not conversion.successful:
        handle_option_on_validation_error(conversion, name)
    else:
        flow.config[name] = conversion.result
    return flow.config[name]

def make_component(
    name,
    on_new_form = on_new_form,
    on_validation_error = on_validation_error,
    on_valid_data = on_valid_data,
    on_generate_form_html = on_generate_form_html,
    on_generate_head_html = on_generate_head_html,
):

    def run(flow):
        flow['ext'] = AttributeDict(
            on_generate_form_html=on_generate_form_html,
            on_generate_head_html=on_generate_head_html,
        )
        option=dict(
            to=[(unicode(i), to.name) for i, to in enumerate(flow.config.ext.to)]
        )
        if not flow.http.input:
            form = CaptchaForm(
                value=dict(
                    to=u'0'
                ),
                option=option,
            )
            flow.ext['form'] = form
            flow.http.response.body = [on_new_form(flow)]
        else:
            # Handle the submission
            conversion = Conversion(
                flow.http.input
            ).perform(
                multiDictToDict(encoding='utf8'),
                flow,
            )
            if not conversion.successful:
                raise Exception(conversion.error)
            params = conversion.result
            conversion = Conversion(
                params
            ).perform(
                contact_form_converter,
                flow,
            )
            if not conversion.successful:
                form = CaptchaForm(
                    value=conversion.value, 
                    error=encode_error(conversion),
                    option=option,
                )
                flow.ext['form'] = form
                flow.http.response.body = [on_validation_error(flow, form)]
            else:
                flow.http.response.body = [on_valid_data(
                    flow,
                    conversion.result,
                )]

    required_services = ['http.input', 'template']
    available_services = {
        'http.input': HTTPInputService,
        'mail': MailService,
        'template': DreamweaverTemplateService,
    }

    def on_load_component(flow):
        flow.config[name] = config(flow, name)
        #if flow.config.has_key('ext'):
        #    raise Exception('Extension configuration is already present')
        #else:
        flow.config['ext'] = flow.config[name]
        def dispatch_contact_form(flow):
            f = flow.copy()
            del f['provider']
            start_flow(
                run,
                provider = ServiceProvider(
                    existing_services = f,
                    available_services = available_services,
                ),
                required_services=required_services,
            )
        return dispatch_contact_form
    return on_load_component

